@@ -1,24 +1,17 @@
-from __future__ import unicode_literals
-
 import datetime
 import decimal
 from collections import defaultdict
 
-from django.contrib.auth import get_permission_codename
 from django.core.exceptions import FieldDoesNotExist
-from django.db import models
+from django.db import models, router
 from django.db.models.constants import LOOKUP_SEP
 from django.db.models.deletion import Collector
-from django.db.models.sql.constants import QUERY_TERMS
 from django.forms.utils import pretty_name
 from django.urls import NoReverseMatch, reverse
-from django.utils import formats, six, timezone
-from django.utils.encoding import force_str, force_text, smart_text
+from django.utils import formats, timezone
 from django.utils.html import format_html
 from django.utils.text import capfirst
-from django.utils.translation import (
-    override as translation_override, ungettext,
-)
+from django.utils.translation import ngettext, override as translation_override
 
 
 class FieldIsAForeignKeyColumnName(Exception):
@@ -28,40 +21,39 @@ class FieldIsAForeignKeyColumnName(Exception):
 
 def lookup_needs_distinct(opts, lookup_path):
     """
-    Returns True if 'distinct()' should be used to query the given lookup path.
+    Return True if 'distinct()' should be used to query the given lookup path.
     """
     lookup_fields = lookup_path.split(LOOKUP_SEP)
-    # Remove the last item of the lookup path if it is a query term
-    if lookup_fields[-1] in QUERY_TERMS:
-        lookup_fields = lookup_fields[:-1]
-    # Now go through the fields (following all relations) and look for an m2m
+    # Go through the fields (following all relations) and look for an m2m.
     for field_name in lookup_fields:
         if field_name == 'pk':
             field_name = opts.pk.name
-        field = opts.get_field(field_name)
-        if hasattr(field, 'get_path_info'):
-            # This field is a relation, update opts to follow the relation
-            path_info = field.get_path_info()
-            opts = path_info[-1].to_opts
-            if any(path.m2m for path in path_info):
-                # This field is a m2m relation so we know we need to call distinct
-                return True
+        try:
+            field = opts.get_field(field_name)
+        except FieldDoesNotExist:
+            # Ignore query lookups.
+            continue
+        else:
+            if hasattr(field, 'get_path_info'):
+                # This field is a relation; update opts to follow the relation.
+                path_info = field.get_path_info()
+                opts = path_info[-1].to_opts
+                if any(path.m2m for path in path_info):
+                    # This field is a m2m relation so distinct must be called.
+                    return True
     return False
 
 
 def prepare_lookup_value(key, value):
     """
-    Returns a lookup value prepared to be used in queryset filtering.
+    Return a lookup value prepared to be used in queryset filtering.
     """
     # if key ends with __in, split parameter into separate values
     if key.endswith('__in'):
         value = value.split(',')
     # if key ends with __isnull, special case '' and the string literals 'false' and '0'
-    if key.endswith('__isnull'):
-        if value.lower() in ('', 'false', '0'):
-            value = False
-        else:
-            value = True
+    elif key.endswith('__isnull'):
+        value = value.lower() not in ('', 'false', '0')
     return value
 
 
@@ -69,10 +61,10 @@ def quote(s):
     """
     Ensure that primary key values do not confuse the admin URLs by escaping
     any '/', '_' and ':' and similarly problematic characters.
-    Similar to urllib.quote, except that the quoting is slightly different so
-    that it doesn't get automatically unquoted by the Web browser.
+    Similar to urllib.parse.quote(), except that the quoting is slightly
+    different so that it doesn't get automatically unquoted by the Web browser.
     """
-    if not isinstance(s, six.string_types):
+    if not isinstance(s, str):
         return s
     res = list(s)
     for i in range(len(res)):
@@ -83,9 +75,7 @@ def quote(s):
 
 
 def unquote(s):
-    """
-    Undo the effects of quote(). Based heavily on urllib.unquote().
-    """
+    """Undo the effects of quote(). Based heavily on urllib.parse.unquote()."""
     mychr = chr
     myatoi = int
     list = s.split('_')
@@ -104,8 +94,9 @@ def unquote(s):
 
 
 def flatten(fields):
-    """Returns a list which is a single level of flattening of the
-    original list."""
+    """
+    Return a list which is a single level of flattening of the original list.
+    """
     flat = []
     for field in fields:
         if isinstance(field, (list, tuple)):
@@ -116,7 +107,7 @@ def flatten(fields):
 
 
 def flatten_fieldsets(fieldsets):
-    """Returns a list of field names from an admin fieldsets structure."""
+    """Return a list of field names from an admin fieldsets structure."""
     field_names = []
     for name, opts in fieldsets:
         field_names.extend(
@@ -125,40 +116,44 @@ def flatten_fieldsets(fieldsets):
     return field_names
 
 
-def get_deleted_objects(objs, opts, user, admin_site, using):
+def get_deleted_objects(objs, request, admin_site):
     """
     Find all objects related to ``objs`` that should also be deleted. ``objs``
     must be a homogeneous iterable of objects (e.g. a QuerySet).
 
-    Returns a nested list of strings suitable for display in the
+    Return a nested list of strings suitable for display in the
     template with the ``unordered_list`` filter.
     """
+    try:
+        obj = objs[0]
+    except IndexError:
+        return [], {}, set(), []
+    else:
+        using = router.db_for_write(obj._meta.model)
     collector = NestedObjects(using=using)
     collector.collect(objs)
     perms_needed = set()
 
     def format_callback(obj):
-        has_admin = obj.__class__ in admin_site._registry
+        model = obj.__class__
+        has_admin = model in admin_site._registry
         opts = obj._meta
 
-        no_edit_link = '%s: %s' % (capfirst(opts.verbose_name),
-                                   force_text(obj))
+        no_edit_link = '%s: %s' % (capfirst(opts.verbose_name), obj)
 
         if has_admin:
+            if not admin_site._registry[model].has_delete_permission(request, obj):
+                perms_needed.add(opts.verbose_name)
             try:
                 admin_url = reverse('%s:%s_%s_change'
                                     % (admin_site.name,
                                        opts.app_label,
                                        opts.model_name),
-                                    None, (quote(obj._get_pk_val()),))
+                                    None, (quote(obj.pk),))
             except NoReverseMatch:
                 # Change url doesn't exist -- don't display link to edit
                 return no_edit_link
 
-            p = '%s.%s' % (opts.app_label,
-                           get_permission_codename('delete', opts))
-            if not user.has_perm(p):
-                perms_needed.add(opts.verbose_name)
             # Display a link to the admin page.
             return format_html('{}: <a href="{}">{}</a>',
                                capfirst(opts.verbose_name),
@@ -179,7 +174,7 @@ def get_deleted_objects(objs, opts, user, admin_site, using):
 
 class NestedObjects(Collector):
     def __init__(self, *args, **kwargs):
-        super(NestedObjects, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.edges = {}  # {from_instance: [to_instances]}
         self.protected = set()
         self.model_objs = defaultdict(set)
@@ -199,12 +194,12 @@ class NestedObjects(Collector):
                 self.add_edge(None, obj)
             self.model_objs[obj._meta.model].add(obj)
         try:
-            return super(NestedObjects, self).collect(objs, source_attr=source_attr, **kwargs)
+            return super().collect(objs, source_attr=source_attr, **kwargs)
         except models.ProtectedError as e:
             self.protected.update(e.protected_objects)
 
     def related_objects(self, related, objs):
-        qs = super(NestedObjects, self).related_objects(related, objs)
+        qs = super().related_objects(related, objs)
         return qs.select_related(related.field.name)
 
     def _nested(self, obj, seen, format_callback):
@@ -254,8 +249,8 @@ def model_format_dict(obj):
     else:
         opts = obj
     return {
-        'verbose_name': force_text(opts.verbose_name),
-        'verbose_name_plural': force_text(opts.verbose_name_plural)
+        'verbose_name': opts.verbose_name,
+        'verbose_name_plural': opts.verbose_name_plural,
     }
 
 
@@ -274,7 +269,7 @@ def model_ngettext(obj, n=None):
         obj = obj.model
     d = model_format_dict(obj)
     singular, plural = d["verbose_name"], d["verbose_name_plural"]
-    return ungettext(singular, plural, n or 0)
+    return ngettext(singular, plural, n or 0)
 
 
 def lookup_field(name, obj, model_admin=None):
@@ -287,10 +282,7 @@ def lookup_field(name, obj, model_admin=None):
         if callable(name):
             attr = name
             value = attr(obj)
-        elif (model_admin is not None and
-                hasattr(model_admin, name) and
-                not name == '__str__' and
-                not name == '__unicode__'):
+        elif hasattr(model_admin, name) and name != '__str__':
             attr = getattr(model_admin, name)
             value = attr(obj)
         else:
@@ -329,11 +321,11 @@ def _get_non_gfk_field(opts, name):
 
 def label_for_field(name, model, model_admin=None, return_attr=False):
     """
-    Returns a sensible label for a field name. The name can be a callable,
-    property (but not created with @property decorator) or the name of an
-    object's attribute, as well as a genuine fields. If return_attr is
-    True, the resolved attribute (which could be a callable) is also returned.
-    This will be None if (and only if) the name refers to a field.
+    Return a sensible label for a field name. The name can be a callable,
+    property (but not created with @property decorator), or the name of an
+    object's attribute, as well as a model field. If return_attr is True, also
+    return the resolved attribute (which could be a callable). This will be
+    None if (and only if) the name refers to a field.
     """
     attr = None
     try:
@@ -344,16 +336,13 @@ def label_for_field(name, model, model_admin=None, return_attr=False):
             # field is likely a ForeignObjectRel
             label = field.related_model._meta.verbose_name
     except FieldDoesNotExist:
-        if name == "__unicode__":
-            label = force_text(model._meta.verbose_name)
-            attr = six.text_type
-        elif name == "__str__":
-            label = force_str(model._meta.verbose_name)
-            attr = bytes
+        if name == "__str__":
+            label = str(model._meta.verbose_name)
+            attr = str
         else:
             if callable(name):
                 attr = name
-            elif model_admin is not None and hasattr(model_admin, name):
+            elif hasattr(model_admin, name):
                 attr = getattr(model_admin, name)
             elif hasattr(model, name):
                 attr = getattr(model, name)
@@ -395,7 +384,7 @@ def help_text_for_field(name, model):
     else:
         if hasattr(field, 'help_text'):
             help_text = field.help_text
-    return smart_text(help_text)
+    return help_text
 
 
 def display_for_field(value, field, empty_value_display):
@@ -403,9 +392,9 @@ def display_for_field(value, field, empty_value_display):
 
     if getattr(field, 'flatchoices', None):
         return dict(field.flatchoices).get(value, empty_value_display)
-    # NullBooleanField needs special-case null-handling, so it comes
-    # before the general null test.
-    elif isinstance(field, models.BooleanField) or isinstance(field, models.NullBooleanField):
+    # BooleanField needs special-case null-handling, so it comes before the
+    # general null test.
+    elif isinstance(field, models.BooleanField):
         return _boolean_icon(value)
     elif value is None:
         return empty_value_display
@@ -430,16 +419,18 @@ def display_for_value(value, empty_value_display, boolean=False):
         return _boolean_icon(value)
     elif value is None:
         return empty_value_display
+    elif isinstance(value, bool):
+        return str(value)
     elif isinstance(value, datetime.datetime):
         return formats.localize(timezone.template_localtime(value))
     elif isinstance(value, (datetime.date, datetime.time)):
         return formats.localize(value)
-    elif isinstance(value, six.integer_types + (decimal.Decimal, float)):
+    elif isinstance(value, (int, decimal.Decimal, float)):
         return formats.number_format(value)
     elif isinstance(value, (list, tuple)):
-        return ', '.join(force_text(v) for v in value)
+        return ', '.join(str(v) for v in value)
     else:
-        return force_text(value)
+        return str(value)
 
 
 class NotRelationField(Exception):
@@ -522,23 +513,23 @@ def construct_change_message(form, formsets, add):
                 for added_object in formset.new_objects:
                     change_message.append({
                         'added': {
-                            'name': force_text(added_object._meta.verbose_name),
-                            'object': force_text(added_object),
+                            'name': str(added_object._meta.verbose_name),
+                            'object': str(added_object),
                         }
                     })
                 for changed_object, changed_fields in formset.changed_objects:
                     change_message.append({
                         'changed': {
-                            'name': force_text(changed_object._meta.verbose_name),
-                            'object': force_text(changed_object),
+                            'name': str(changed_object._meta.verbose_name),
+                            'object': str(changed_object),
                             'fields': changed_fields,
                         }
                     })
                 for deleted_object in formset.deleted_objects:
                     change_message.append({
                         'deleted': {
-                            'name': force_text(deleted_object._meta.verbose_name),
-                            'object': force_text(deleted_object),
+                            'name': str(deleted_object._meta.verbose_name),
+                            'object': str(deleted_object),
                         }
                     })
     return change_message
