@@ -235,10 +235,8 @@ class BaseDatabaseSchemaEditor:
         # If it's a callable, call it
         if callable(default):
             default = default()
-        # Run it through the field's get_db_prep_save method so we can send it
-        # to the database.
-        default = field.get_db_prep_save(default, self.connection)
-        return default
+        # Convert the value so it can be sent to the database.
+        return field.get_db_prep_save(default, self.connection)
 
     def quote_value(self, value):
         """
@@ -478,7 +476,7 @@ class BaseDatabaseSchemaEditor:
         # Reset connection if required
         if self.connection.features.connection_persists_old_columns:
             self.connection.close()
-        # Remove all deferred statements referencing the deleted table.
+        # Remove all deferred statements referencing the deleted column.
         for sql in list(self.deferred_sql):
             if isinstance(sql, Statement) and sql.references_column(model._meta.db_table, field.column):
                 self.deferred_sql.remove(sql)
@@ -541,9 +539,9 @@ class BaseDatabaseSchemaEditor:
                 fks_dropped.add((old_field.column,))
                 self.execute(self._delete_constraint_sql(self.sql_delete_fk, model, fk_name))
         # Has unique been removed?
-        if old_field.unique and (not new_field.unique or (not old_field.primary_key and new_field.primary_key)):
+        if old_field.unique and (not new_field.unique or self._field_became_primary_key(old_field, new_field)):
             # Find the unique constraint for this field
-            constraint_names = self._constraint_names(model, [old_field.column], unique=True)
+            constraint_names = self._constraint_names(model, [old_field.column], unique=True, primary_key=False)
             if strict and len(constraint_names) != 1:
                 raise ValueError("Found wrong number (%s) of unique constraints for %s.%s" % (
                     len(constraint_names),
@@ -586,12 +584,11 @@ class BaseDatabaseSchemaEditor:
             # db_index=True.
             index_names = self._constraint_names(model, [old_field.column], index=True, type_=Index.suffix)
             for index_name in index_names:
-                if index_name in meta_index_names:
+                if index_name not in meta_index_names:
                     # The only way to check if an index was created with
                     # db_index=True or with Index(['field'], name='foo')
                     # is to look at its name (refs #28053).
-                    continue
-                self.execute(self._delete_constraint_sql(self.sql_delete_index, model, index_name))
+                    self.execute(self._delete_constraint_sql(self.sql_delete_index, model, index_name))
         # Change check constraints?
         if old_db_params['check'] != new_db_params['check'] and old_db_params['check']:
             constraint_names = self._constraint_names(model, [old_field.column], check=True)
@@ -688,10 +685,11 @@ class BaseDatabaseSchemaEditor:
         if post_actions:
             for sql, params in post_actions:
                 self.execute(sql, params)
+        # If primary_key changed to False, delete the primary key constraint.
+        if old_field.primary_key and not new_field.primary_key:
+            self._delete_primary_key(model, strict)
         # Added a unique?
-        if (not old_field.unique and new_field.unique) or (
-            old_field.primary_key and not new_field.primary_key and new_field.unique
-        ):
+        if self._unique_should_be_added(old_field, new_field):
             self.execute(self._create_unique_sql(model, [new_field.column]))
         # Added an index? Add an index if db_index switched to True or a unique
         # constraint will no longer be used in lieu of an index. The following
@@ -710,11 +708,7 @@ class BaseDatabaseSchemaEditor:
         if old_field.primary_key and new_field.primary_key and old_type != new_type:
             rels_to_update.extend(_related_non_m2m_objects(old_field, new_field))
         # Changed to become primary key?
-        # Note that we don't detect unsetting of a PK, as we assume another field
-        # will always come along and replace it.
-        if not old_field.primary_key and new_field.primary_key:
-            # First, drop the old PK
-            self._delete_primary_key(model, strict)
+        if self._field_became_primary_key(old_field, new_field):
             # Make the new one
             self.execute(
                 self.sql_create_pk % {
@@ -881,8 +875,7 @@ class BaseDatabaseSchemaEditor:
         and a unique digest and suffix.
         """
         _, table_name = split_identifier(table_name)
-        hash_data = [table_name] + list(column_names)
-        hash_suffix_part = '%s%s' % (self._digest(*hash_data), suffix)
+        hash_suffix_part = '%s%s' % (self._digest(table_name, *column_names), suffix)
         max_length = self.connection.ops.max_name_length() or 200
         # If everything fits into max_length, use that name.
         index_name = '%s_%s_%s' % (table_name, '_'.join(column_names), hash_suffix_part)
@@ -971,6 +964,14 @@ class BaseDatabaseSchemaEditor:
     def _field_should_be_indexed(self, model, field):
         return field.db_index and not field.unique
 
+    def _field_became_primary_key(self, old_field, new_field):
+        return not old_field.primary_key and new_field.primary_key
+
+    def _unique_should_be_added(self, old_field, new_field):
+        return (not old_field.unique and new_field.unique) or (
+            old_field.primary_key and not new_field.primary_key and new_field.unique
+        )
+
     def _rename_field_sql(self, table, old_field, new_field, new_type):
         return self.sql_rename_column % {
             "table": self.quote_name(table),
