@@ -178,7 +178,7 @@ class MigrationGraph:
         except KeyError as err:
             raise NodeNotFoundError(
                 "Unable to find replacement node %r. It was either never added"
-                " to the migration graph, or has been removed." % (replacement, ),
+                " to the migration graph, or has been removed." % (replacement,),
                 replacement
             ) from err
         for replaced_key in replaced:
@@ -214,7 +214,7 @@ class MigrationGraph:
         except KeyError as err:
             raise NodeNotFoundError(
                 "Unable to remove replacement node %r. It was either never added"
-                " to the migration graph, or has been removed already." % (replacement, ),
+                " to the migration graph, or has been removed already." % (replacement,),
                 replacement
             ) from err
         replaced_nodes = set()
@@ -256,7 +256,7 @@ class MigrationGraph:
         follow if applying the migrations to a database.
         """
         if target not in self.nodes:
-            raise NodeNotFoundError("Node %r not a valid node" % (target, ), target)
+            raise NodeNotFoundError("Node %r not a valid node" % (target,), target)
         # Use parent.key instead of parent to speed up the frequent hashing in ensure_not_cyclic
         self.ensure_not_cyclic(target, lambda x: (parent.key for parent in self.node_map[x].parents))
         self.cached = True
@@ -275,7 +275,7 @@ class MigrationGraph:
         would follow if removing the migrations from a database.
         """
         if target not in self.nodes:
-            raise NodeNotFoundError("Node %r not a valid node" % (target, ), target)
+            raise NodeNotFoundError("Node %r not a valid node" % (target,), target)
         # Use child.key instead of child to speed up the frequent hashing in ensure_not_cyclic
         self.ensure_not_cyclic(target, lambda x: (child.key for child in self.node_map[x].children))
         self.cached = True
@@ -304,7 +304,7 @@ class MigrationGraph:
         """
         roots = set()
         for node in self.nodes:
-            if not any(key[0] == node[0] for key in self.node_map[node].parents) and (not app or app == node[0]):
+            if all(key[0] != node[0] for key in self.node_map[node].parents) and (not app or app == node[0]):
                 roots.add(node)
         return sorted(roots)
 
@@ -318,7 +318,7 @@ class MigrationGraph:
         """
         leaves = set()
         for node in self.nodes:
-            if not any(key[0] == node[0] for key in self.node_map[node].children) and (not app or app == node[0]):
+            if all(key[0] != node[0] for key in self.node_map[node].children) and (not app or app == node[0]):
                 leaves.add(node)
         return sorted(leaves)
 
@@ -352,6 +352,14 @@ class MigrationGraph:
     def _nodes_and_edges(self):
         return len(self.nodes), sum(len(node.parents) for node in self.node_map.values())
 
+    def _generate_plan(self, nodes, at_end):
+        plan = []
+        for node in nodes:
+            for migration in self.forwards_plan(node):
+                if migration not in plan and (at_end or migration not in nodes):
+                    plan.append(migration)
+        return plan
+
     def make_state(self, nodes=None, at_end=True, real_apps=None):
         """
         Given a migration node or nodes, return a complete ProjectState for it.
@@ -360,17 +368,11 @@ class MigrationGraph:
         """
         if nodes is None:
             nodes = list(self.leaf_nodes())
-        if len(nodes) == 0:
+        if not nodes:
             return ProjectState()
         if not isinstance(nodes[0], tuple):
             nodes = [nodes]
-        plan = []
-        for node in nodes:
-            for migration in self.forwards_plan(node):
-                if migration not in plan:
-                    if not at_end and migration in nodes:
-                        continue
-                    plan.append(migration)
+        plan = self._generate_plan(nodes, at_end)
         project_state = ProjectState(real_apps=real_apps)
         for node in plan:
             project_state = self.nodes[node].mutate_state(project_state, preserve=False)
