@@ -1,10 +1,8 @@
-from __future__ import absolute_import
-
 import inspect
 import warnings
 
 
-class RemovedInDjango20Warning(PendingDeprecationWarning):
+class RemovedInDjango30Warning(PendingDeprecationWarning):
     pass
 
 
@@ -12,7 +10,7 @@ class RemovedInNextVersionWarning(DeprecationWarning):
     pass
 
 
-class warn_about_renamed_method(object):
+class warn_about_renamed_method:
     def __init__(self, class_name, old_method_name, new_method_name, deprecation_warning):
         self.class_name = class_name
         self.old_method_name = old_method_name
@@ -44,7 +42,7 @@ class RenameMethodsBase(type):
     renamed_methods = ()
 
     def __new__(cls, name, bases, attrs):
-        new_class = super(RenameMethodsBase, cls).__new__(cls, name, bases, attrs)
+        new_class = super().__new__(cls, name, bases, attrs)
 
         for base in inspect.getmro(new_class):
             class_name = base.__name__
@@ -78,59 +76,19 @@ class DeprecationInstanceCheck(type):
             "`%s` is deprecated, use `%s` instead." % (self.__name__, self.alternative),
             self.deprecation_warning, 2
         )
-        return super(DeprecationInstanceCheck, self).__instancecheck__(instance)
+        return super().__instancecheck__(instance)
 
 
-class CallableBool:
-    """
-    An boolean-like object that is also callable for backwards compatibility.
-    """
-    do_not_call_in_templates = True
-
-    def __init__(self, value):
-        self.value = value
-
-    def __bool__(self):
-        return self.value
-
-    def __call__(self):
-        warnings.warn(
-            "Using user.is_authenticated() and user.is_anonymous() as a method "
-            "is deprecated. Remove the parentheses to use it as an attribute.",
-            RemovedInDjango20Warning, stacklevel=2
-        )
-        return self.value
-
-    def __nonzero__(self):  # Python 2 compatibility
-        return self.value
-
-    def __repr__(self):
-        return 'CallableBool(%r)' % self.value
-
-    def __eq__(self, other):
-        return self.value == other
-
-    def __ne__(self, other):
-        return self.value != other
-
-    def __or__(self, other):
-        return bool(self.value or other)
-
-CallableFalse = CallableBool(False)
-CallableTrue = CallableBool(True)
-
-
-class MiddlewareMixin(object):
+class MiddlewareMixin:
     def __init__(self, get_response=None):
         self.get_response = get_response
-        super(MiddlewareMixin, self).__init__()
+        super().__init__()
 
     def __call__(self, request):
         response = None
         if hasattr(self, 'process_request'):
             response = self.process_request(request)
-        if not response:
-            response = self.get_response(request)
+        response = response or self.get_response(request)
         if hasattr(self, 'process_response'):
             response = self.process_response(request, response)
         return response
